//
//  JPCompanyJobView.m
//  Portfolio: GitHub Jobs
//
//  Created by Julius Parishy on 5/2/12.
//  Copyright (c) 2012 DiDev Studios. All rights reserved.
//

#import "GHJobListingView.h"

// http://stackoverflow.com/questions/277055/remove-html-tags-from-an-nsstring-on-the-iphone

@interface NSString (StripHTML)

-(NSString *) stringByStrippingHTML;

@end

@implementation NSString (StripHTML)

-(NSString *) stringByStrippingHTML {
    NSRange r;
    NSString *s = [self copy];
    while ((r = [s rangeOfString:@"<[^>]+>" options:NSRegularExpressionSearch]).location != NSNotFound)
        s = [s stringByReplacingCharactersInRange:r withString:@""];
    return s; }

@end

@interface GHJobListingView ()

-(void)downloadCompanyLogoWithURL:(NSURL*)url;
-(void)updateLogoImageViewWithImage:(UIImage*)image;

-(void)buttonOverlayPressed;

@end

@implementation GHJobListingView

@synthesize jobListing;

@synthesize companyNameLabel;
@synthesize companyPositionLabel;

@synthesize companyLogoImageView;

@synthesize companyPositionDescriptionLabel;

@synthesize button;

@synthesize pressedBlock;

-(id)initWithFrame:(CGRect)frame jobListing:(GHJobListing *)listing pressedBlock:(GHJobListingPressedBlock)pressed
{
    if((self = [super initWithFrame:frame]) != nil)
    {
        self.jobListing = listing;
        
        self.pressedBlock = pressed;
        
        [self configureViewProperites];
        
        self.companyNameLabel.text = self.jobListing.company;
        self.companyPositionLabel.text = self.jobListing.title;
        
        CGRect originalFrame = self.companyPositionDescriptionLabel.frame;
        
        self.companyPositionDescriptionLabel.text = [self.jobListing.description stringByStrippingHTML];
        self.companyPositionDescriptionLabel.numberOfLines = 0;
        self.companyPositionDescriptionLabel.clipsToBounds = YES;
        [self.companyPositionDescriptionLabel sizeToFit];
        
        CGRect adjustedFrame = self.companyPositionDescriptionLabel.frame;
        adjustedFrame.size.height = originalFrame.size.height;
        self.companyPositionDescriptionLabel.frame = adjustedFrame;
        
        if(self.jobListing.companyLogoURL != nil)
            [self downloadCompanyLogoWithURL:self.jobListing.companyLogoURL];
    }
    
    return self;
}

-(void) configureViewProperites
{
    self.backgroundColor = [UIColor colorWithWhite:1.0f alpha:0.7f];
    self.clipsToBounds = YES;
    
    UIFont* companyNameFont = [UIFont fontWithName:@"Helvetica Neue" size:14.0f];
    UIFont* companyDetailFont = [UIFont fontWithName:@"Helvetica Neue" size:12.0f];
   
    // Company Name
    CGRect companyNameFrame = CGRectMake(10.0f, 10.0f, self.frame.size.width - 20.0f, 16.0f);
    companyNameLabel = [[UILabel alloc] initWithFrame:companyNameFrame];
    
    self.companyNameLabel.font = companyNameFont;
    self.companyNameLabel.textColor = [UIColor darkTextColor];
    self.companyNameLabel.backgroundColor = [UIColor clearColor];
    
    [self addSubview:self.companyNameLabel];
    
    // Company Position
    CGRect companyPositionFrame = CGRectMake(10.0f, companyNameFrame.origin.y + companyNameFrame.size.height + 0.0f, self.frame.size.width - 20.0f, 16.0f);
    companyPositionLabel = [[UILabel alloc] initWithFrame:companyPositionFrame];
    
    self.companyPositionLabel.font = companyDetailFont;
    self.companyPositionLabel.textColor = [UIColor darkGrayColor];
    self.companyPositionLabel.backgroundColor = [UIColor clearColor];
    
    [self addSubview:self.companyPositionLabel];
    
    // Company Logo
    //CGRect companyLogoImageViewFrame = CGRectMake(self.frame.size.width - 10.0f, 10.0f, 100.0f, 75.0f);
    CGRect companyLogoImageViewFrame = CGRectMake(10.0f, 10.0f, 100.0f, 75.0f);
    companyLogoImageViewFrame.origin.x -= companyLogoImageViewFrame.size.width;
    
    companyLogoImageView = [[UIImageView alloc] initWithFrame:companyLogoImageViewFrame];
    self.companyLogoImageView.alpha = 0.0f;
    
    [self addSubview:self.companyLogoImageView];
    
    // Company Position Description
    float companyPositionDescriptionY = companyPositionFrame.origin.y + companyPositionFrame.size.height + 5.0f;
    CGRect companyPositionDescriptionFrame = CGRectMake(10.0f, companyPositionDescriptionY, self.frame.size.width - 20.0f, self.frame.size.height - companyPositionDescriptionY - 10.0f);
    companyPositionDescriptionLabel = [[UILabel alloc] initWithFrame:companyPositionDescriptionFrame];
    
    self.companyPositionDescriptionLabel.font = companyDetailFont;
    self.companyPositionDescriptionLabel.textColor = [UIColor darkTextColor];
    self.companyPositionDescriptionLabel.backgroundColor = [UIColor clearColor];
    
    [self addSubview:self.companyPositionDescriptionLabel];
    
    // Ze invisibel button
    button = [[UIButton alloc] initWithFrame:self.bounds];
    [self.button addTarget:self action:@selector(buttonOverlayPressed) forControlEvents:UIControlEventTouchUpInside];
    [self addSubview:button];
}

-(void)updateLogoImageViewWithImage:(UIImage*)image
{
    self.jobListing.companyLogoImage = image;
    
    // Calculate the width/height
    const float maximumWidth = 90.0f;
    const float maximumHeight = 40.0f;
    
    float imageWidth = 0.0f;
    float imageHeight = 0.0f;
    
    // Resize and reserve aspect ratio

    float imageAspectRatio = image.size.width / image.size.height;
    
    imageHeight = MIN(image.size.height, maximumHeight);
    imageWidth = imageHeight * imageAspectRatio;

    if(imageWidth > maximumWidth)
    {
        imageAspectRatio = image.size.height / image.size.width;
        
        imageWidth = maximumWidth;
        imageHeight = imageWidth * imageAspectRatio;
    }

    // Fix the frame
    CGRect frame = self.companyLogoImageView.frame;
    frame.size = CGSizeMake(imageWidth, imageHeight);
    //frame.origin.x = self.frame.size.width - imageWidth - 10.0f;
    frame.origin.x = 10.0f;
    frame.origin.y = 45.0f;
    
    // Set the image but hide it with alpha of 0
    self.companyLogoImageView.alpha = 0.0f;
    self.companyLogoImageView.image = image;
    self.companyLogoImageView.frame = frame;
    
    self.companyPositionDescriptionLabel.alpha = 0.0f;
    
    // Animate the description into the right spot
    [UIView animateWithDuration:0.5f animations:^{
       
        CGRect descriptionFrame = self.companyPositionDescriptionLabel.frame;
        
        float difference = frame.size.height + 5.0f;
        descriptionFrame.origin.y += difference;
        descriptionFrame.size.height -= difference;
        
        [self.companyPositionDescriptionLabel sizeToFit];
        self.companyPositionDescriptionLabel.frame = descriptionFrame;
        
        self.companyPositionDescriptionLabel.alpha = 1.0f;
        
    } completion:^(BOOL finished) {
        
        // And finally set the alpha
        [UIView animateWithDuration:0.5f animations:^{
            self.companyLogoImageView.alpha = 0.45f;
        }];
    }];
}

-(void) downloadCompanyLogoWithURL:(NSURL *)url
{
    
    NSURLRequest* logoRequest = [NSURLRequest requestWithURL:url];
    [NSURLConnection sendAsynchronousRequest:logoRequest queue:[NSOperationQueue mainQueue] completionHandler:^(NSURLResponse* response, NSData* data, NSError* error) {
       
        if(error != nil)
        {
            // Use no logo image
            NSLog(@"Failed to download logo.");
        }
        else
        {
            UIImage* image = [UIImage imageWithData:data];
            if(image != nil)
            {
                [self performSelectorOnMainThread:@selector(updateLogoImageViewWithImage:) withObject:image waitUntilDone:NO];
            }
            else
            {
                NSString* returnedString = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
                NSLog(@"Data returned was not an image:\n%@", returnedString);
            }
        }
    }];
}

-(void)buttonOverlayPressed
{
    GHJobListingPressedBlock block = self.pressedBlock;
    block(self.jobListing);
}

@end
